"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function groupBy(items, column) {
    const keys = [...new Set(items.map(t => t[column]))];
    const dict = {};
    keys.forEach(k => dict[k] = []);
    items.forEach(t => {
        const key = t[column];
        dict[key].push(t);
    });
    return dict;
}
exports.groupBy = groupBy;
function equalAsSets(a, b) {
    if (a.length !== b.length) {
        return false;
    }
    return setsEqual(new Set(a), new Set(b));
}
exports.equalAsSets = equalAsSets;
function setsEqual(aSet, bSet) {
    if (aSet.size !== bSet.size) {
        return false;
    }
    return [...aSet].every(value => bSet.has(value));
}
exports.setsEqual = setsEqual;
function graphqlCompliantToValue(value) {
    if (!value) {
        return value;
    }
    const translations = {
        "AO__": "Å",
        "AE__": "Ä",
        "OE__": "Ö"
    };
    Object.entries(translations).forEach(t => {
        while (value.indexOf(t[0]) !== -1) {
            value = value.replace(t[0], t[1]);
        }
    });
    return value;
}
exports.graphqlCompliantToValue = graphqlCompliantToValue;
function valueToGraphqlCompliant(value) {
    if (!value) {
        return value;
    }
    const translations = {
        "Å": "AO__",
        "Ä": "AE__",
        "Ö": "OE__",
    };
    Object.entries(translations).forEach(t => {
        while (value.indexOf(t[0]) !== -1) {
            value = value.replace(t[0], t[1]);
        }
    });
    return value;
}
exports.valueToGraphqlCompliant = valueToGraphqlCompliant;
//# sourceMappingURL=misc.js.map